** Estimation of intervention effects on fold increae in 
** interval-censored data at baseline and end-of-trial

** Based on expression (3) in Xu et al. A simple and powerful method for the 
** estimation of intervention effects ... J Biopharm Stat

** Example, hypothetical data that resembling HI assay data 
** non-detect (below first dilution level 10) is indicated by 5
** 10 means standard titre 10 and true titre censored between 10 and 20, etc.
** 640 is higest dilution and true titer censored between 640 and +infinity
** t=0 and t=1 represent baseline and end-of-trial
** x=1 for intervention group at time t=1, otherwise x=0
 
version 12.1

clear
set more off

use example_HI,clear

list id treat titre t x if id==9 | id==10 | id==39 | id==40

** LY and UY are the Lower and Upper boundary of the intervals, log2 scale

cap drop TL
cap drop TU
cap drop LY
cap drop UY
gen TL=cond(titre==5,.,titre)
gen TU=cond(titre==640, .,titre*2)
gen LY=(ln(TL/10)/ln(2))+1
gen UY=(ln(TU/10)/ln(2))+1

list id treat titre t x LY UY if id==9 | id==10 | id==39 | id==40
 
** logC=2 indicates 4-fold increase for the log2 transformed data
local logC=2

** fit model  
xtset id
xtintreg LY UY x t, i(id) 

** non-linear combination of model parameters 
qui{
nlcom (r1b: ([LY]_b[t]+[LY]_b[x]-`logC')/[sigma_e]_b[_cons]/sqrt(2)) ///
 (r0b: ([LY]_b[t]-`logC')/[sigma_e]_b[_cons]/sqrt(2))  ///
 (lnrr: ln(normal(([LY]_b[t]+[LY]_b[x]-`logC')/(sqrt(2)*[sigma_e]_b[_cons])))-ln(normal(([LY]_b[t]-`logC')/(sqrt(2)*[sigma_e]_b[_cons]))))  ///
 (rd: -normal(([LY]_b[t]+[LY]_b[x]-`logC')/(sqrt(2)*[sigma_e]_b[_cons]))+normal(([LY]_b[t]-`logC')/(sqrt(2)*[sigma_e]_b[_cons])))

* retrieve point estimates and confidence intervals
mat b=r(b)
mat V=r(V)

local r1=normal(b[1,1])
local r1_ub=normal(b[1,1]+1.96*sqrt(V[1,1]))
local r1_lb=normal(b[1,1]-1.96*sqrt(V[1,1]))

local r0=normal(b[1,2])
local r0_ub=normal(b[1,2]+1.96*sqrt(V[2,2]))
local r0_lb=normal(b[1,2]-1.96*sqrt(V[2,2]))

local rr=exp(b[1,3])
local rr_ub=exp(b[1,3]+1.96*sqrt(V[3,3]))
local rr_lb=exp(b[1,3]-1.96*sqrt(V[3,3]))

local rd=b[1,4]
local rd_ub=b[1,4]+1.96*sqrt(V[4,4])
local rd_lb=b[1,4]-1.96*sqrt(V[4,4])

mat a=(`r0', `r0_lb', `r0_ub')\(`r1', `r1_lb', `r1_ub')\(`rd', `rd_lb', `rd_ub')\(`rr', `rr_lb', `rr_ub')
mat rownames a=r0 r1 RD RR
mat colnames a=est lb_95%CI ub_95%CI
}
disp "Fold-increase analysis, using expression (3)"
matlist a
 


** Based on expression (4) in Xu et al. A simple and powerful method for the 
** estimation of intervention effects ... J Biopharm Stat

** logC=2 indicates 4-fold increase for the log2 transformed data
** logb1=3 indicates the minimum required value at end-of-study after log2 transformation
local logC=2
local logb1=3


** fit model
cap xtset, clear
xtset id
xtintreg LY UY x t, i(id) 

** non-linear combination of model parameters 
qui{
nlcom (r1b: normal(([LY]_b[t]+[LY]_b[x]-`logC')/[sigma_e]_b[_cons]/sqrt(2)) ///
   -binormal(([LY]_b[t]+[LY]_b[x]-`logC')/[sigma_e]_b[_cons]/sqrt(2), ///
   (`logb1'-[LY]_b[_cons]-[LY]_b[t]-[LY]_b[x])/sqrt([sigma_e]_b[_cons]^2+[sigma_u]_b[_cons]^2),  ///
   -[sigma_e]_b[_cons]/sqrt([sigma_e]_b[_cons]^2+[sigma_u]_b[_cons]^2)/sqrt(2))) /// 
(r0b: normal(([LY]_b[t]-`logC')/[sigma_e]_b[_cons]/sqrt(2)) ///
   -binormal(([LY]_b[t]-`logC')/[sigma_e]_b[_cons]/sqrt(2), ///
   (`logb1'-[LY]_b[_cons]-[LY]_b[t])/sqrt([sigma_e]_b[_cons]^2+[sigma_u]_b[_cons]^2),  ///
   -[sigma_e]_b[_cons]/sqrt([sigma_e]_b[_cons]^2+[sigma_u]_b[_cons]^2)/sqrt(2)))   /// 
(lnrr: ln(normal(([LY]_b[t]+[LY]_b[x]-`logC')/[sigma_e]_b[_cons]/sqrt(2)) ///
   -binormal(([LY]_b[t]+[LY]_b[x]-`logC')/[sigma_e]_b[_cons]/sqrt(2), ///
   (`logb1'-[LY]_b[_cons]-[LY]_b[t]-[LY]_b[x])/sqrt([sigma_e]_b[_cons]^2+[sigma_u]_b[_cons]^2),  ///
   -[sigma_e]_b[_cons]/sqrt([sigma_e]_b[_cons]^2+[sigma_u]_b[_cons]^2)/sqrt(2))) ///   
   -ln(normal(([LY]_b[t]-`logC')/[sigma_e]_b[_cons]/sqrt(2)) ///
   -binormal(([LY]_b[t]-`logC')/[sigma_e]_b[_cons]/sqrt(2), ///
   (`logb1'-[LY]_b[_cons]-[LY]_b[t])/sqrt([sigma_e]_b[_cons]^2+[sigma_u]_b[_cons]^2),  ///
   -[sigma_e]_b[_cons]/sqrt([sigma_e]_b[_cons]^2+[sigma_u]_b[_cons]^2)/sqrt(2)) ))   ///  
 (rd:  normal(([LY]_b[t]-`logC')/[sigma_e]_b[_cons]/sqrt(2)) ///
   -binormal(([LY]_b[t]-`logC')/[sigma_e]_b[_cons]/sqrt(2), ///
   (`logb1'-[LY]_b[_cons]-[LY]_b[t])/sqrt([sigma_e]_b[_cons]^2+[sigma_u]_b[_cons]^2),  ///
   -[sigma_e]_b[_cons]/sqrt([sigma_e]_b[_cons]^2+[sigma_u]_b[_cons]^2)/sqrt(2))   ///
 -(normal(([LY]_b[t]+[LY]_b[x]-`logC')/[sigma_e]_b[_cons]/sqrt(2)) ///
   -binormal(([LY]_b[t]+[LY]_b[x]-`logC')/[sigma_e]_b[_cons]/sqrt(2), ///
   (`logb1'-[LY]_b[_cons]-[LY]_b[t]-[LY]_b[x])/sqrt([sigma_e]_b[_cons]^2+[sigma_u]_b[_cons]^2),  ///
   -[sigma_e]_b[_cons]/sqrt([sigma_e]_b[_cons]^2+[sigma_u]_b[_cons]^2)/sqrt(2)))  )
 
* retrieve point estimates and confidence intervals 
mat b=r(b)
mat V=r(V)

local r1=b[1,1]
local r1_ub=b[1,1]+1.96*sqrt(V[1,1])
local r1_lb=b[1,1]-1.96*sqrt(V[1,1])

local r0=b[1,2]
local r0_ub=(b[1,2]+1.96*sqrt(V[2,2]))
local r0_lb=(b[1,2]-1.96*sqrt(V[2,2]))

local rr=exp(b[1,3])
local rr_ub=exp(b[1,3]+1.96*sqrt(V[3,3]))
local rr_lb=exp(b[1,3]-1.96*sqrt(V[3,3]))

local rd=b[1,4]
local rd_ub=b[1,4]+1.96*sqrt(V[4,4])
local rd_lb=b[1,4]-1.96*sqrt(V[4,4])

mat a=(`r0', `r0_lb', `r0_ub')\(`r1', `r1_lb', `r1_ub')\(`rd', `rd_lb', `rd_ub')\(`rr', `rr_lb', `rr_ub')
mat rownames a=r0 r1 RD RR
mat colnames a=est lb_95%CI ub_95%CI
}
disp "Fold-increase analysis, using expression (4)"
matlist a 
 
 
 
 
